-- Do not use this file except in compliance with the License. You may
-- obtain a copy of the License at http://www.microcore.org/License/
-- Software distributed under the License is distributed on an "AS IS" basis,
-- WITHOUT WARRANTY OF ANY KIND, either express or implied.
-- See the License for the specific language governing rights and limitations
-- under the License.
--
-- The Original Code is: MicroCore DEBUG_ROM.VHD 1.10
--
-- The Initial Developer of the Original Code is Klaus.Schleisiek AT hamburg.de.
--
-- Introduction  see: information/uCore_Introduction.pdf
-- Tools         see: information/uCore_Implementation.pdf
-- Versions      see: uCore_Versions.txt
-- License       see: information/uCore_License.pdf
-- Conventions   see: information/uCore_Conventions.pdf
-- Contributors  see: information/uCore_Contributors.pdf
-- Philosophy    see: information/uCore_Philosophy.pdf
--
-- This is a model for an asynchronous RAM, optimised for simulation. 
-- It serves as program ROM and can be (re)loaded via the umbilical
-- while the core is on halt via CLK_EN.

LIBRARY IEEE;
USE     IEEE.std_logic_1164.ALL;
USE     work.functions.ALL;
USE     work.constants.ALL;

-- The whole RAM interface is contained in RECORD dram_port
ENTITY debug_rom IS
PORT (debugROM : INOUT debugrom_port);
END debug_rom;

ARCHITECTURE sim_model OF debug_rom IS

TYPE rom_data IS ARRAY ((2 ** prog_addr_width)-1 DOWNTO 0) OF inst_bus;

SIGNAL clk  : std_logic; 
SIGNAL addr : prog_addr;

BEGIN
-----------------------------------------------------------------------
-- ram_model:
--       Input data are taken in iff clk edge & WE = 1
-----------------------------------------------------------------------

clk  <= debugROM.clk;  -- for the synthesizer
addr <= debugROM.addr; -- for the synthesizer

debug_rom: PROCESS(clk, addr)
VARIABLE rom : rom_data;
BEGIN
  IF  rising_edge(clk)  THEN
     IF  debugROM.write='1'  THEN
         rom(to_int('0' & addr)) := debugROM.din;
     END IF;
  END IF;

-- Read rom
  debugROM.dout <= rom(to_int('0' & addr));

END PROCESS debug_rom;

END sim_model;
