\ Do not use this file except in compliance with the License. You may
\ obtain a copy of the License at http://www.microcore.org/License/
\ Software distributed under the License is distributed on an "AS IS" basis,
\ WITHOUT WARRANTY OF ANY KIND, either express or implied.
\ See the License for the specific language governing rights and limitations
\ under the License.
\
\ The Original Code is: MicroCore CORETEST.F 1.10
\
\ The Initial Developer of the Original Code is Klaus.Schleisiek AT hamburg.de.
\
\ Introduction  see: information/uCore_Introduction.pdf
\ Tools         see: information/uCore_Implementation.pdf
\ Versions      see: uCross_Versions.txt
\ License       see: information/uCore_License.pdf
\ Conventions   see: information/uCore_Conventions.pdf
\ Contributors  see: information/uCore_Contributors.pdf
\ Philosophy    see: information/uCore_Philosophy.pdf
\
\ MicroCore core test for VHDL-Version 1.10
\
forget Image  Create Image     \ prune dicitionary and compile bye marker IMAGE again, so TEST.SRC can be loaded several times
New                            \ reset target compiler
Target                         \ switch into target compilation mode and context

include constants.f            \ MicroCore Register addresses and bits

#pause 8 + code-origin
         0 data-origin

Variable Location  1 allot

: set_carry    ( -- ) status@ #c  or status! ;
: reset_carry  ( -- ) status@ [ #c invert ] Literal and status! ;

: zeroEXIT   ( n1 -- n2 )     dup  ?EXIT  0= ;
: zero-EXIT  ( n1 -- n2 )     dup 0=EXIT  0= ;
: signEXIT   ( n1 -- n2 )    sign  ?EXIT  0= ;
: sign-EXIT  ( n1 -- n2 )    sign 0=EXIT  0= ;
: carryEXIT  ( n1 -- n2 )   carry  ?EXIT  0= ;
: carry-EXIT ( n1 -- n2 )   carry 0=EXIT  0= ;

: test_branches  ( -- ) 
         0 sign IF  invert  THEN    IF  $1 BEGIN REPEAT THEN
        -1 sign IF  invert  THEN    IF  $2 BEGIN REPEAT THEN
    0 u2/ carry IF  invert  THEN    IF  $3 BEGIN REPEAT THEN
    1 u2/ carry IF  invert  THEN 0= IF  $4 BEGIN REPEAT THEN
        -1  zeroEXIT invert         IF  $5 BEGIN REPEAT THEN
         0  zeroEXIT invert         IF  $6 BEGIN REPEAT THEN
        -1 zero-EXIT                IF  $7 BEGIN REPEAT THEN
         0 zero-EXIT                IF  $8 BEGIN REPEAT THEN
        -1  signEXIT invert         IF  $9 BEGIN REPEAT THEN
         0  signEXIT invert         IF  $A BEGIN REPEAT THEN
         0 sign-EXIT                IF  $B BEGIN REPEAT THEN
        -1 sign-EXIT                IF  $C BEGIN REPEAT THEN
    1 u2/  carryEXIT                IF  $D BEGIN REPEAT THEN
    0 u2/  carryEXIT invert         IF  $E BEGIN REPEAT THEN
    0 u2/ carry-EXIT                IF  $F BEGIN REPEAT THEN
    1 u2/ carry-EXIT invert         IF $10 BEGIN REPEAT THEN
;

\ : smallest  ( -- min )   -1 u2/ invert ;  \ sets carry bit
: smallest  ( -- min )  0 rol -2 or u2/ invert ; \ carry bit unaffected

: test_unary ( -- )
   -1 2* invert 1 = 0=                 IF  $12 BEGIN REPEAT THEN  ( -- )
                              cc carry IF  $13 BEGIN REPEAT THEN  ( -- )
   -1 u2/                     cc carry IF  $14 BEGIN REPEAT THEN  ( $7FF )
   invert 2*                  cc carry IF  $15 BEGIN REPEAT THEN  (    0 )
   dup 0= invert                       IF  $16 BEGIN REPEAT THEN  (    0 )
   cc ror                        carry IF  $17 BEGIN REPEAT THEN  ( $800 )
   smallest  -                         IF  $18 BEGIN REPEAT THEN  ( -- )
   smallest rol               cc carry IF  $19 BEGIN REPEAT THEN  (    1 )
   rol                           carry IF  $1A BEGIN REPEAT THEN  ( -- )
   2 -                                 IF  $1B BEGIN REPEAT THEN  ( -- )
   0 0= invert  1 0= or                IF  $1C BEGIN REPEAT THEN  ( -- )
   smallest 2/   smallest dup u2/ or - IF  $1D BEGIN REPEAT THEN  ( -- )
   1 2/                       cc carry IF  $1E BEGIN REPEAT THEN  (    0 )
   0= invert                           IF  $1F BEGIN REPEAT THEN  ( -- )
;
\ SL     BOTH ALU Op: 2*       2*
\ ASR    BOTH ALU Op: 2/       2/
\ LSR    BOTH ALU Op: u2/      u2/
\ ROR    BOTH ALU Op: ror      dont
\ ROL    BOTH ALU Op: rol      dont
\ ZEQU   BOTH ALU Op: 0=       0=
\ CC     BOTH ALU Op: cc       dont

: 0<  ( n1 -- f )      dup sign IF  invert  THEN  xor ;
: d+  ( d1 d2 -- d3 )  >r swap >r +  r> r> +c ;
: d-  ( d1 d2 -- d3 )  >r swap >r -  r> r> -c ;

: test_arith  ( -- )
   3 4 +  ( 7 ) 4 -  ( 3 ) 4 2dup_+  ( 3 4 7 ) over_swap_-  
   ( 3 4 -3 )  dup 0< and  ( 3 4 -3 )  negate
   ( 3 4 3 ) or  ( 3 7 ) xor  ( 4 ) 4 and  ( 4 ) 
   0 -1 0 d+  ( 3 1 )   3 1 d-  or 
   0=EXIT $20 BEGIN REPEAT
;

: test_memory  ( -- )
   1 2   Location  
   0 ++! 1 ++!  -1 ++@  1 ++@ drop  
   2* - 0=EXIT $30 BEGIN REPEAT
;
: modify  ( n -- /n )  0= ;

: absbranch  ( n -- )   ;

: test_?call  ( -- )
              0 ?CALL modify    IF  $41 BEGIN REPEAT THEN
              1 ?CALL modify    IF  $42 BEGIN REPEAT THEN
             0 0=CALL modify 0= IF  $43 BEGIN REPEAT THEN
             1 0=CALL modify 0= IF  $44 BEGIN REPEAT THEN
   0 u2/ carry  ?CALL modify    IF  $45 BEGIN REPEAT THEN
   1 u2/ carry  ?CALL modify 0= IF  $46 BEGIN REPEAT THEN
   0 u2/ carry 0=CALL modify 0= IF  $47 BEGIN REPEAT THEN
   1 u2/ carry 0=CALL modify    IF  $48 BEGIN REPEAT THEN
       -1 sign  ?CALL modify    IF  $49 BEGIN REPEAT THEN
        0 sign  ?CALL modify    IF  $4A BEGIN REPEAT THEN
       -1 sign 0=CALL modify 0= IF  $4B BEGIN REPEAT THEN
        0 sign 0=CALL modify 0= IF  $4C BEGIN REPEAT THEN
  -1 ['] modify    nop CALL     IF  $4D BEGIN REPEAT THEN
     ['] absbranch nop BRANCH       $4E BEGIN REPEAT
;
: test_stack ( -- )
   $55 dup +   $55 2* -                   IF  $50 BEGIN REPEAT THEN
   1 2 swap 1 - >r 2 - r> or              IF  $51 BEGIN REPEAT THEN
   1 2 over 1 - >r 2 - >r 1 - r> or r> or IF  $52 BEGIN REPEAT THEN
   1 2 3 drop 2 - swap 1 - or             IF  $53 BEGIN REPEAT THEN
\   test_stack_depths - not implemented
;
: test_register ( -- )
   1 >r r@ r@ + 2 -                           IF  $60 BEGIN REPEAT THEN
   $55 r> + $56 -  r@ 1 = or                  IF  $61 BEGIN REPEAT THEN
   status@  0 status! status@                 IF  $62 BEGIN REPEAT THEN
   dup status!  status@ - #zero -             IF  $63 BEGIN REPEAT THEN
   1 2 3 dup dup dsp@ 1- dsp! drop drop + 3 - IF  $64 BEGIN REPEAT THEN
   1 >r 2 >r rsp@ 1+ rsp! r> 1-               IF  $65 BEGIN REPEAT THEN
;
: test_iobus  ( -- )
   $99 ioport1 !   $71 ioport2 !
   ioport1 @ $99 = INVERT  ioport2 @ $99 =  OR IF  $70 BEGIN REPEAT THEN
;
: test_local  ( -- )
   r@   0 l@ -                                   IF  $80 BEGIN REPEAT THEN
   r@   -1 0 l!  r@  over 0 l!  0 l@ swap 1+ - - IF  $81 BEGIN REPEAT THEN
        5 -80 ++l!  @  5 -                       IF  $82 BEGIN REPEAT THEN
;
: test_tasks  ( -- )
   TASK_REG @   $100 TASK_REG !
   TASK_REG @ $100 - +       IF  $90 BEGIN REPEAT THEN
   6 4 ++u! @ 6 -            IF  $90 BEGIN REPEAT THEN
   -6 4 u!  4 u@ -6 -        IF  $91 BEGIN REPEAT THEN
;

: test  ( -- )   
   1 2 3 0        -1 Ioport1 ! 
   test_register   1 Ioport1 !
   test_stack      2 Ioport1 !
   test_?call      3 Ioport1 !
   test_branches   4 Ioport1 !
   test_unary      5 Ioport1 !
   test_arith      6 Ioport1 !
   test_memory     7 Ioport1 !
   test_iobus      8 Ioport1 !
   test_local      9 Ioport1 !
   test_tasks     10 Ioport1 !
   + - + IF  $100 BEGIN REPEAT THEN
                   0 ioport1 !       \ everything is ok if there is a zero in IoPort1 and a zero on the data stack
   0 BEGIN REPEAT
;

here #reset code-origin ] ['] test nop BRANCH [ code-origin    \ compile branch to TEST at reset vector location
here #irq   code-origin ]           IRET      [ code-origin
here #pause code-origin ] r> 1- >r  IRET      [ code-origin    \ retry instruction

Host                                               \ back into the Host environment

